@@ -35,6 +35,7 @@ from simpletransformers.classification.transformer_models.xlm_model import XLMFo
 from simpletransformers.classification.transformer_models.xlm_roberta_model import XLMRobertaForSequenceClassification
 from simpletransformers.classification.transformer_models.xlnet_model import XLNetForSequenceClassification
 from simpletransformers.config.global_args import global_args
+from simpletransformers.classification.classification_utils import LazyClassificationDataset
 from simpletransformers.custom_models.models import ElectraForSequenceClassification
 from tensorboardX import SummaryWriter
 from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
@@ -54,6 +55,9 @@ from transformers import (
     ElectraTokenizer,
     FlaubertConfig,
     FlaubertTokenizer,
+    LongformerConfig,
+    LongformerForSequenceClassification,
+    LongformerTokenizer,
     RobertaConfig,
     RobertaTokenizer,
     XLMConfig,
@@ -95,16 +99,17 @@ class ClassificationModel:
         """  # noqa: ignore flake8"
 
         MODEL_CLASSES = {
+            "albert": (AlbertConfig, AlbertForSequenceClassification, AlbertTokenizer),
             "bert": (BertConfig, BertForSequenceClassification, BertTokenizer),
+            "camembert": (CamembertConfig, CamembertForSequenceClassification, CamembertTokenizer),
+            "distilbert": (DistilBertConfig, DistilBertForSequenceClassification, DistilBertTokenizer),
+            "electra": (ElectraConfig, ElectraForSequenceClassification, ElectraTokenizer),
+            "flaubert": (FlaubertConfig, FlaubertForSequenceClassification, FlaubertTokenizer),
+            "longformer": (LongformerConfig, LongformerForSequenceClassification, LongformerTokenizer),
+            "roberta": (RobertaConfig, RobertaForSequenceClassification, RobertaTokenizer),
             "xlnet": (XLNetConfig, XLNetForSequenceClassification, XLNetTokenizer),
             "xlm": (XLMConfig, XLMForSequenceClassification, XLMTokenizer),
-            "roberta": (RobertaConfig, RobertaForSequenceClassification, RobertaTokenizer),
-            "distilbert": (DistilBertConfig, DistilBertForSequenceClassification, DistilBertTokenizer),
-            "albert": (AlbertConfig, AlbertForSequenceClassification, AlbertTokenizer),
-            "camembert": (CamembertConfig, CamembertForSequenceClassification, CamembertTokenizer),
             "xlmroberta": (XLMRobertaConfig, XLMRobertaForSequenceClassification, XLMRobertaTokenizer),
-            "flaubert": (FlaubertConfig, FlaubertForSequenceClassification, FlaubertTokenizer),
-            "electra": (ElectraConfig, ElectraForSequenceClassification, ElectraTokenizer),
         }
 
         if args and "manual_seed" in args:
@@ -119,6 +124,12 @@ class ClassificationModel:
             "tie_value": 1,
             "stride": 0.8,
             "regression": False,
+            "lazy_text_column": 0,
+            "lazy_text_a_column": None,
+            "lazy_text_b_column": None,
+            "lazy_labels_column": 1,
+            "lazy_header_row": True,
+            "lazy_delimiter": "\t",
         }
 
         self.args.update(global_args)
@@ -234,33 +245,42 @@ class ClassificationModel:
 
         self._move_model_to_device()
 
-        if "text" in train_df.columns and "labels" in train_df.columns:
-            train_examples = [
-                InputExample(i, text, None, label)
-                for i, (text, label) in enumerate(zip(train_df["text"], train_df["labels"]))
-            ]
-        elif "text_a" in train_df.columns and "text_b" in train_df.columns:
-            train_examples = [
-                InputExample(i, text_a, text_b, label)
-                for i, (text_a, text_b, label) in enumerate(
-                    zip(train_df["text_a"], train_df["text_b"], train_df["labels"])
-                )
-            ]
+        if isinstance(train_df, str):
+            if self.args["sliding_window"]:
+                raise ValueError("Lazy loading cannot be used with sliding window.")
+            train_dataset = LazyClassificationDataset(train_df, self.tokenizer, self.args)
         else:
-            warnings.warn(
-                "Dataframe headers not specified. Falling back to using column 0 as text and column 1 as labels."
-            )
-            train_examples = [
-                InputExample(i, text, None, label)
-                for i, (text, label) in enumerate(zip(train_df.iloc[:, 0], train_df.iloc[:, 1]))
-            ]
+            if "text" in train_df.columns and "labels" in train_df.columns:
+                train_examples = [
+                    InputExample(i, text, None, label)
+                    for i, (text, label) in enumerate(zip(train_df["text"].astype(str), train_df["labels"]))
+                ]
+            elif "text_a" in train_df.columns and "text_b" in train_df.columns:
+                train_examples = [
+                    InputExample(i, text_a, text_b, label)
+                    for i, (text_a, text_b, label) in enumerate(
+                        zip(train_df["text_a"].astype(str), train_df["text_b"].astype(str), train_df["labels"])
+                    )
+                ]
+            else:
+                warnings.warn(
+                    "Dataframe headers not specified. Falling back to using column 0 as text and column 1 as labels."
+                )
+                train_examples = [
+                    InputExample(i, text, None, label)
+                    for i, (text, label) in enumerate(zip(train_df.iloc[:, 0], train_df.iloc[:, 1]))
+                ]
 
-        train_dataset = self.load_and_cache_examples(train_examples, verbose=verbose)
+            train_dataset = self.load_and_cache_examples(train_examples, verbose=verbose)
+        train_sampler = RandomSampler(train_dataset)
+        train_dataloader = DataLoader(
+            train_dataset, sampler=train_sampler, batch_size=self.args["train_batch_size"], num_workers=14
+        )
 
         os.makedirs(output_dir, exist_ok=True)
 
         global_step, tr_loss = self.train(
-            train_dataset,
+            train_dataloader,
             output_dir,
             multi_label=multi_label,
             show_running_loss=show_running_loss,
@@ -280,7 +300,7 @@ class ClassificationModel:
 
     def train(
         self,
-        train_dataset,
+        train_dataloader,
         output_dir,
         multi_label=False,
         show_running_loss=True,
@@ -294,13 +314,10 @@ class ClassificationModel:
         Utility function to be used by the train_model() method. Not intended to be used directly.
         """
 
-        device = self.device
         model = self.model
         args = self.args
 
         tb_writer = SummaryWriter(logdir=args["tensorboard_dir"])
-        train_sampler = RandomSampler(train_dataset)
-        train_dataloader = DataLoader(train_dataset, sampler=train_sampler, batch_size=args["train_batch_size"])
 
         t_total = len(train_dataloader) // args["gradient_accumulation_steps"] * args["num_train_epochs"]
 
@@ -383,7 +400,6 @@ class ClassificationModel:
                 if steps_trained_in_current_epoch > 0:
                     steps_trained_in_current_epoch -= 1
                     continue
-                batch = tuple(t.to(device) for t in batch)
 
                 inputs = self._get_inputs_dict(batch)
                 outputs = model(**inputs)
@@ -453,7 +469,7 @@ class ClassificationModel:
                         results, _, _ = self.eval_model(
                             eval_df,
                             verbose=verbose and args["evaluate_during_training_verbose"],
-                            silent=True,
+                            silent=args["evaluate_during_training_silent"],
                             **kwargs,
                         )
                         for key, value in results.items():
@@ -545,7 +561,10 @@ class ClassificationModel:
 
             if args["evaluate_during_training"]:
                 results, _, _ = self.eval_model(
-                    eval_df, verbose=verbose and args["evaluate_during_training_verbose"], silent=True, **kwargs
+                    eval_df,
+                    verbose=verbose and args["evaluate_during_training_verbose"],
+                    silent=args["evaluate_during_training_silent"],
+                    **kwargs,
                 )
 
                 self._save_model(output_dir_current, optimizer, scheduler, results=results)
@@ -645,40 +664,44 @@ class ClassificationModel:
         Utility function to be used by the eval_model() method. Not intended to be used directly.
         """
 
-        device = self.device
         model = self.model
         args = self.args
         eval_output_dir = output_dir
 
         results = {}
-
-        if "text" in eval_df.columns and "labels" in eval_df.columns:
-            eval_examples = [
-                InputExample(i, text, None, label)
-                for i, (text, label) in enumerate(zip(eval_df["text"], eval_df["labels"]))
-            ]
-        elif "text_a" in eval_df.columns and "text_b" in eval_df.columns:
-            eval_examples = [
-                InputExample(i, text_a, text_b, label)
-                for i, (text_a, text_b, label) in enumerate(
-                    zip(eval_df["text_a"], eval_df["text_b"], eval_df["labels"])
+        if isinstance(eval_df, str):
+            eval_dataset = LazyClassificationDataset(eval_df, self.tokenizer, self.args)
+            eval_examples = None
+        else:
+            if "text" in eval_df.columns and "labels" in eval_df.columns:
+                eval_examples = [
+                    InputExample(i, text, None, label)
+                    for i, (text, label) in enumerate(zip(eval_df["text"].astype(str), eval_df["labels"]))
+                ]
+            elif "text_a" in eval_df.columns and "text_b" in eval_df.columns:
+                eval_examples = [
+                    InputExample(i, text_a, text_b, label)
+                    for i, (text_a, text_b, label) in enumerate(
+                        zip(eval_df["text_a"].astype(str), eval_df["text_b"].astype(str), eval_df["labels"])
+                    )
+                ]
+            else:
+                warnings.warn(
+                    "Dataframe headers not specified. Falling back to using column 0 as text and column 1 as labels."
                 )
-            ]
-        else:
-            warnings.warn(
-                "Dataframe headers not specified. Falling back to using column 0 as text and column 1 as labels."
-            )
-            eval_examples = [
-                InputExample(i, text, None, label)
-                for i, (text, label) in enumerate(zip(eval_df.iloc[:, 0], eval_df.iloc[:, 1]))
-            ]
+                eval_examples = [
+                    InputExample(i, text, None, label)
+                    for i, (text, label) in enumerate(zip(eval_df.iloc[:, 0], eval_df.iloc[:, 1]))
+                ]
 
-        if args["sliding_window"]:
-            eval_dataset, window_counts = self.load_and_cache_examples(
-                eval_examples, evaluate=True, verbose=verbose, silent=silent
-            )
-        else:
-            eval_dataset = self.load_and_cache_examples(eval_examples, evaluate=True, verbose=verbose, silent=silent)
+            if args["sliding_window"]:
+                eval_dataset, window_counts = self.load_and_cache_examples(
+                    eval_examples, evaluate=True, verbose=verbose, silent=silent
+                )
+            else:
+                eval_dataset = self.load_and_cache_examples(
+                    eval_examples, evaluate=True, verbose=verbose, silent=silent
+                )
         os.makedirs(eval_output_dir, exist_ok=True)
 
         eval_sampler = SequentialSampler(eval_dataset)
@@ -691,7 +714,7 @@ class ClassificationModel:
         model.eval()
 
         for batch in tqdm(eval_dataloader, disable=args["silent"] or silent):
-            batch = tuple(t.to(device) for t in batch)
+            # batch = tuple(t.to(device) for t in batch)
 
             with torch.no_grad():
                 inputs = self._get_inputs_dict(batch)
@@ -813,7 +836,7 @@ class ClassificationModel:
                 sep_token=tokenizer.sep_token,
                 # RoBERTa uses an extra separator b/w pairs of sentences,
                 # cf. github.com/pytorch/fairseq/commit/1684e166e3da03f5b600dbb7855cb98ddfcd0805
-                sep_token_extra=bool(args["model_type"] in ["roberta", "camembert", "xlmroberta"]),
+                sep_token_extra=bool(args["model_type"] in ["roberta", "camembert", "xlmroberta", "longformer"]),
                 # PAD on the left for XLNet
                 pad_on_left=bool(args["model_type"] in ["xlnet"]),
                 pad_token=tokenizer.convert_tokens_to_ids([tokenizer.pad_token])[0],
@@ -825,6 +848,7 @@ class ClassificationModel:
                 sliding_window=args["sliding_window"],
                 flatten=not evaluate,
                 stride=args["stride"],
+                add_prefix_space=bool(args["model_type"] in ["roberta", "camembert", "xlmroberta", "longformer"]),
                 args=args,
             )
             if verbose and args["sliding_window"]:
@@ -853,7 +877,7 @@ class ClassificationModel:
         else:
             return dataset
 
-    def compute_metrics(self, preds, labels, eval_examples, multi_label=False, **kwargs):
+    def compute_metrics(self, preds, labels, eval_examples=None, multi_label=False, **kwargs):
         """
         Computes the evaluation metrics for the model predictions.
 
@@ -877,7 +901,10 @@ class ClassificationModel:
 
         mismatched = labels != preds
 
-        wrong = [i for (i, v) in zip(eval_examples, mismatched) if v.any()]
+        if eval_examples:
+            wrong = [i for (i, v) in zip(eval_examples, mismatched) if v.any()]
+        else:
+            wrong = ["NA"]
 
         if multi_label:
             label_ranking_score = label_ranking_average_precision_score(labels, preds)
@@ -908,7 +935,6 @@ class ClassificationModel:
             model_outputs: A python list of the raw model outputs for each text.
         """
 
-        device = self.device
         model = self.model
         args = self.args
 
@@ -941,7 +967,7 @@ class ClassificationModel:
         if self.config.output_hidden_states:
             for batch in tqdm(eval_dataloader, disable=args["silent"]):
                 model.eval()
-                batch = tuple(t.to(device) for t in batch)
+                # batch = tuple(t.to(device) for t in batch)
 
                 with torch.no_grad():
                     inputs = self._get_inputs_dict(batch)
@@ -959,19 +985,23 @@ class ClassificationModel:
                 if preds is None:
                     preds = logits.detach().cpu().numpy()
                     out_label_ids = inputs["labels"].detach().cpu().numpy()
-                    all_layer_hidden_states = [state.detach().cpu().numpy() for state in layer_hidden_states]
+                    all_layer_hidden_states = np.array([state.detach().cpu().numpy() for state in layer_hidden_states])
                     all_embedding_outputs = embedding_outputs.detach().cpu().numpy()
                 else:
                     preds = np.append(preds, logits.detach().cpu().numpy(), axis=0)
                     out_label_ids = np.append(out_label_ids, inputs["labels"].detach().cpu().numpy(), axis=0)
                     all_layer_hidden_states = np.append(
-                        [state.detach().cpu().numpy() for state in layer_hidden_states], axis=0
+                        all_layer_hidden_states,
+                        np.array([state.detach().cpu().numpy() for state in layer_hidden_states]),
+                        axis=1,
+                    )
+                    all_embedding_outputs = np.append(
+                        all_embedding_outputs, embedding_outputs.detach().cpu().numpy(), axis=0
                     )
-                    all_embedding_outputs = np.append(embedding_outputs.detach().cpu().numpy(), axis=0)
         else:
             for batch in tqdm(eval_dataloader, disable=args["silent"]):
                 model.eval()
-                batch = tuple(t.to(device) for t in batch)
+                # batch = tuple(t.to(device) for t in batch)
 
                 with torch.no_grad():
                     inputs = self._get_inputs_dict(batch)
@@ -1045,11 +1075,17 @@ class ClassificationModel:
         self.model.to(self.device)
 
     def _get_inputs_dict(self, batch):
-        inputs = {"input_ids": batch[0], "attention_mask": batch[1], "labels": batch[3]}
+        if isinstance(batch[0], dict):
+            inputs = {key: value.squeeze().to(self.device) for key, value in batch[0].items()}
+            inputs["labels"] = batch[1].to(self.device)
+        else:
+            batch = tuple(t.to(self.device) for t in batch)
 
-        # XLM, DistilBERT and RoBERTa don't use segment_ids
-        if self.args["model_type"] != "distilbert":
-            inputs["token_type_ids"] = batch[2] if self.args["model_type"] in ["bert", "xlnet", "albert"] else None
+            inputs = {"input_ids": batch[0], "attention_mask": batch[1], "labels": batch[3]}
+
+            # XLM, DistilBERT and RoBERTa don't use segment_ids
+            if self.args["model_type"] != "distilbert":
+                inputs["token_type_ids"] = batch[2] if self.args["model_type"] in ["bert", "xlnet", "albert"] else None
 
         return inputs
 
