@@ -134,6 +134,13 @@ class ConvAIModel:
 
         self.args.update(global_args)
 
+        saved_model_args = self._load_model_args(model_name)
+        if saved_model_args:
+            self.args.update(saved_model_args)
+
+        if args:
+            self.args.update(args)
+
         if not use_cuda:
             self.args["fp16"] = False
 
@@ -215,10 +222,7 @@ class ConvAIModel:
             **kwargs,
         )
 
-        model_to_save = self.model.module if hasattr(self.model, "module") else self.model
-        model_to_save.save_pretrained(output_dir)
-        self.tokenizer.save_pretrained(output_dir)
-        torch.save(self.args, os.path.join(output_dir, "training_args.bin"))
+        self._save_model(model=self.model)
 
         if verbose:
             logger.info(" Training of {} model complete. Saved to {}.".format(self.args["model_type"], output_dir))
@@ -366,7 +370,7 @@ class ConvAIModel:
                         results, _, _ = self.eval_model(
                             eval_dataloader,
                             verbose=verbose and args["evaluate_during_training_verbose"],
-                            silent=True,
+                            silent=args["evaluate_during_training_silent"],
                             **kwargs,
                         )
                         for key, value in results.items():
@@ -602,6 +606,7 @@ class ConvAIModel:
             proxies=self.__dict__.get("proxies", None),
             evaluate=evaluate,
             no_cache=no_cache,
+            args=args,
         )
         # logger.info(personachat.keys())
         # datasets = {"train": defaultdict(list), "valid": defaultdict(list)}
@@ -705,6 +710,7 @@ class ConvAIModel:
                 process_count=process_count,
                 proxies=self.__dict__.get("proxies", None),
                 interact=True,
+                args=args,
             )
             personalities = [dialog["personality"] for dataset in dataset.values() for dialog in dataset]
             personality = random.choice(personalities)
@@ -752,14 +758,16 @@ class ConvAIModel:
 
         return training_progress_scores
 
-    def _save_model(self, output_dir, model=None, results=None):
-        os.makedirs(output_dir, exist_ok=True)
+    def _save_model(self, output_dir=None, model=None, results=None):
+        if not output_dir:
+            output_dir = self.args["output_dir"]
 
-        if model:
+        if model and not self.args["no_save"]:
             # Take care of distributed/parallel training
             model_to_save = model.module if hasattr(model, "module") else model
             model_to_save.save_pretrained(output_dir)
             self.tokenizer.save_pretrained(output_dir)
+            self._save_model_args(output_dir)
 
         if results:
             output_eval_file = os.path.join(output_dir, "eval_results.txt")
@@ -869,3 +877,15 @@ class ConvAIModel:
             current_output.append(prev.item())
 
         return current_output
+
+    def _save_model_args(self, output_dir):
+        os.makedirs(output_dir, exist_ok=True)
+        with open(os.path.join(output_dir, "model_args.json"), "w") as f:
+            json.dump(self.args, f)
+
+    def _load_model_args(self, input_dir):
+        model_args_file = os.path.join(input_dir, "model_args.json")
+        if os.path.isfile(model_args_file):
+            with open(model_args_file, "r") as f:
+                model_args = json.load(f)
+            return model_args
