@@ -55,6 +55,9 @@ from transformers import (
     DistilBertTokenizer,
     ElectraConfig,
     ElectraTokenizer,
+    LongformerConfig,
+    LongformerTokenizer,
+    LongformerForQuestionAnswering,
     RobertaConfig,
     RobertaForQuestionAnswering,
     RobertaTokenizer,
@@ -101,6 +104,7 @@ class QuestionAnsweringModel:
             "bert": (BertConfig, BertForQuestionAnswering, BertTokenizer),
             "distilbert": (DistilBertConfig, DistilBertForQuestionAnswering, DistilBertTokenizer),
             "electra": (ElectraConfig, ElectraForQuestionAnswering, ElectraTokenizer),
+            "longformer": (LongformerConfig, LongformerForQuestionAnswering, LongformerTokenizer),
             "roberta": (RobertaConfig, RobertaForQuestionAnswering, RobertaTokenizer),
             "xlm": (XLMConfig, XLMForQuestionAnswering, XLMTokenizer),
             "xlmroberta": (XLMRobertaConfig, XLMRobertaForQuestionAnswering, XLMRobertaTokenizer),
